#include <NTL/ZZ_pXFactoring.h>
#include <flint/fmpz.h>
#include <flint/fmpz_mod_poly.h>
#include <cstdio>

NTL_CLIENT


struct FlintZZ {
   fmpz_t value;

   explicit
   FlintZZ(const ZZ& a)
   {
      long n = NumBits(a);
   
      fmpz_init(value);
   
      for (long i = 0; i < n; i++)
         if (bit(a, i)) fmpz_setbit(value, i);
   
      if (a < 0)
         fmpz_neg(value, value);

   }

   ~FlintZZ() { fmpz_clear(value); }

};

struct FlintModulus {
   fmpz_mod_ctx_t value;

   explicit FlintModulus(const ZZ& p)
   {
      FlintZZ f_p(p);
      fmpz_mod_ctx_init(value, f_p.value);
   }

   ~FlintModulus()
   {
      fmpz_mod_ctx_clear(value);
   }
};



struct FlintZZ_pX {
   fmpz_mod_poly_t value;
   const FlintModulus& mod;

   FlintZZ_pX(const ZZ_pX& a, const FlintModulus& _mod) : mod(_mod)
   {
      long da = deg(a);
      fmpz_mod_poly_init2(value, da+1, mod.value);

      for (long i = 0; i <= da; i++) {
         FlintZZ f_c(rep(a[i]));
         fmpz_mod_poly_set_coeff_fmpz(value, i, f_c.value, mod.value);
      }
   }

   explicit
   FlintZZ_pX(const FlintModulus& _mod) : mod(_mod)
   {
      fmpz_mod_poly_init(value, mod.value);
   }

   ~FlintZZ_pX() { fmpz_mod_poly_clear(value, mod.value); }
};


struct FlintFac {
   fmpz_mod_poly_factor_t value;
   const FlintModulus& mod;

   explicit
   FlintFac(const FlintModulus& _mod) : mod(_mod) 
   { fmpz_mod_poly_factor_init(value, mod.value); }
   ~FlintFac() { fmpz_mod_poly_factor_clear(value, mod.value); }

};

int main()
{
   setbuf(stdout, NULL);

   for (long l = 256; l <= 4096; l *= 2) {
      // for (long n = 256; n <= 16384; n *= 2) {
      for (long idx = 0; idx < 9; idx ++) {
         long n  = 256*(1L << idx/2);
         if (idx & 1) n += n/2;
         SetSeed((ZZ(l) << 64) + ZZ(n));

	 ZZ p;

	 RandomPrime(p, l);
	 ZZ_p::init(p);

         ZZ_pX f;
      
         random(f, n);
         SetCoeff(f, n);

#if 0
         if (idx == 2) {
           cerr << p << "\n";
           cerr << (n+1) << " ";
           for (long i = 0; i <= n; i++) cerr << f[i] << " ";
           cerr << "\n";
           return 0;
         }
#endif

         Vec< Pair<ZZ_pX, long> > fac;
      
      
         double t;
      
         t = GetTime();
         CanZass(fac, f);
         t = GetTime()-t;
         double NTLTime = t;

         long iter = 1;
         if (NTLTime < 5) {
            t = NTLTime;
            while (t < 5) {
               t += NTLTime;
               iter += 1;
            }

            t = GetTime();
            for (long i = 0; i < iter; i++) CanZass(fac, f);
            t = GetTime()-t;
            NTLTime = t;
         }

         FlintModulus mod(ZZ_p::modulus());
      
         FlintZZ_pX f_f(f,mod);
      
         t = GetTime();
         for (long i = 0; i < iter; i++) {
           FlintFac f_fac(mod);
           fmpz_mod_poly_factor_kaltofen_shoup(f_fac.value, f_f.value, mod.value);
         }
         t = GetTime()-t;
         double FlintTime = t;
      
         printf("%8.2f", FlintTime/NTLTime);
      }

      printf("\n");
   }
}


